@@ -37,6 +37,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <libsoup/soup-address.h>
+#include <glib.h>
 #include <glib/gstdio.h>
 
 #include "gupnp.h"
@@ -336,7 +337,11 @@ gupnp_context_manager_create (guint port)
 #endif
         GUPnPContextManager *impl;
         GType impl_type = G_TYPE_INVALID;
+#ifdef G_OS_WIN32
+#include "gupnp-windows-context-manager.h"
 
+        impl_type = GUPNP_TYPE_WINDOWS_CONTEXT_MANAGER;
+#else
 #ifdef USE_NETWORK_MANAGER
 #include "gupnp-network-manager.h"
         system_bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL);
@@ -360,7 +365,7 @@ gupnp_context_manager_create (guint port)
 #else
                 impl_type = GUPNP_TYPE_UNIX_CONTEXT_MANAGER;
 #endif
-
+#endif /* G_OS_WIN32 */
         impl = g_object_new (impl_type,
                              "port", port,
                              NULL);
@@ -425,7 +430,7 @@ gupnp_context_manager_manage_root_device (GUPnPContextManager *manager,
 guint
 gupnp_context_manager_get_port (GUPnPContextManager *manager)
 {
-        g_return_if_fail (GUPNP_IS_CONTEXT_MANAGER (manager));
+        g_return_val_if_fail (GUPNP_IS_CONTEXT_MANAGER (manager), 0);
 
         return manager->priv->port;
 }
